import es
import cfglib
import cmdlib
import popuplib
import cPickle
import time
import gamethread

#AddonInfo:
info = es.AddonInfo() 
info.name     	= "Hit Indicator" 
info.basename 	= "hitindicator"
info.version  	= "v1.0"
info.url      	= "www.NextGenerationGaming.de" 
info.description = "Mod for hitmarkers and hitsounds"
info.author   	= "Ruuubi"

#Globals:
pathAddon = es.getAddonPath('hitindicator')
pathDB = pathAddon+'/database/playersettings.db'

#Lists:
downloadList = []

#Dicts:
markerDict = {}
soundDict = {}
database = {}
try:
	database = cPickle.load(open(pathDB,"rb"))
except:
	pass

def setDownloads():
	for file in downloadList: es.stringtable('downloadables',file)

def addMarker(args):
	if not enableMarker: return
	global markerDict
	if len(args) >= 2:
		if len(args) < 3:
			critMarker = args[1]
		else:
			critMarker = args[2]
			downloadList.append('materials/'+critMarker+'.vtf')
			downloadList.append('materials/'+critMarker+'.vmt')
		markerDict[args[0]] = {'normal': args[1], 'headshot': critMarker}
		downloadList.append('materials/'+args[1]+'.vtf')
		downloadList.append('materials/'+args[1]+'.vmt')

def addSound(args):
	if not enableSound: return
	global soundDict
	if len(args) >= 2:
		if len(args) < 3:
			critSound = args[1]
		else:
			critSound = args[2]
			downloadList.append('sound/'+critSound)
		soundDict[args[0]] = {'normal': args[1], 'headshot': critSound}
		downloadList.append('sound/'+args[1])

popupTrigger = str(es.ServerVar('hitindicator_commands','!hitsound;!hitsounds;hitsound;hitsounds;!hitmarker;hitmarker','')).split(';')
savetime = es.ServerVar('hitindicator_savetime',31,'')
enableMarker = es.ServerVar('hitindicator_enable_marker',1,'')
useDefaultMarker = es.ServerVar('hitindicator_use_default_marker',1,'')
defaultHitmarker = str(es.ServerVar('hitindicator_default_marker','Call of Duty 2',''))
enableSound = es.ServerVar('hitindicator_enable_sound',1,'')
useDefaultSound = es.ServerVar('hitindicator_use_default_sound',1,'')
defaultHitsound = str(es.ServerVar('hitindicator_default_sound','Quake 3',''))
cmdlib.registerServerCommand('hitindicator_add_marker',addMarker,'')
cmdlib.registerServerCommand('hitindicator_add_sound',addSound,'')
cfglib.AddonCFG(pathAddon+'/configs/config.cfg').execute()

class PlayerManager(dict):
	def __getitem__(self,userid):
		userid = int(userid)
		if userid not in self: self[userid] = PlayerObject(userid)
		return super(PlayerManager,self).__getitem__(userid)

	def __delitem__(self,userid):
		userid = int(userid)
		if userid in self:
			players[userid].saveInDict()
			super(PlayerManager,self).__delitem__(userid)

	def saveAll(self):
		if savetime > 0:
			for userid in self: self[userid].saveInDict()
			cPickle.dump(database,open(pathDB,"wb"))

class PlayerObject(object):
	def __init__(self,userid):
		self.userid = userid
		self.steamid = es.getplayersteamid(userid)
		if self.steamid in database:
			self.hitmarker = database[self.steamid]['hitmarker'] if database[self.steamid]['hitmarker'] in markerDict else None
			self.hitsound = database[self.steamid]['hitsound'] if database[self.steamid]['hitsound'] in soundDict else None
			self.volume = database[self.steamid]['volume']
		else:
			if useDefaultMarker == 1: self.hitmarker = defaultHitmarker if defaultHitmarker in markerDict else None
			else: self.hitmarker = None
			if useDefaultSound == 1: self.hitsound = defaultHitsound if defaultHitsound in soundDict else None
			else: self.hitsound = None
			self.volume = 1.0

	def saveInDict(self):
		global database
		if self.steamid not in database: database[self.steamid] = {}
		database[self.steamid]['hitmarker'] = self.hitmarker
		database[self.steamid]['hitsound'] = self.hitsound
		database[self.steamid]['volume'] = self.volume
		database[self.steamid]['lastcon'] = int(time.time())

class DelayManager(object):
	def __init__(self):
		self.delays = []

	def add(self,userid,duration):
		if userid in self.delays:
			gamethread.cancelDelayed('hitmarkerDelay_%s' % userid)
		else:
			self.delays.append(userid)
		gamethread.delayedname(duration,'hitmarkerDelay_%s' % userid,self.hideHitmarker,userid)

	def hideHitmarker(self,userid):
		if userid in self.delays:
			self.delays.remove(userid)
			es.cexec(userid,"r_screenoverlay 0")

players = PlayerManager()
delay = DelayManager()

def load():
	es.set('hitindicator_version',info.version)
	es.makepublic('hitindicator_version')
	setDownloads()
	if len(database) > 0:
		curTime = int(time.time())
		for player in dict(database):
			if 'lastcon' in database[player]:
				if (curTime-database[player]['lastcon'])/86400 >= savetime: del database[player]
		cPickle.dump(database,open(pathDB,"wb"))
	for userid in es.getUseridList(): whisper(userid,'#blue%s #greensuccessfully loaded!' % info.version)

def unload():
	cmdlib.unregisterServerCommand('hitindicator_add_marker')
	cmdlib.unregisterServerCommand('hitindicator_add_sound')
	players.saveAll()
	for userid in es.getUseridList(): whisper(userid,'#blue%s #greensuccessfully unloaded!' % info.version)

def es_map_start():
	setDownloads()

def player_say(ev):
	if ev['text'] in popupTrigger:
		popupOpen(int(ev['userid']),'main')
		playHintSound(int(ev['userid']),1)

def player_hurt(ev):
	userid = int(ev['userid'])
	attacker = int(ev['attacker'])
	if ev['es_attackersteamid'] == 'BOT': return
	if attacker > 0 and userid != attacker:
		if ev['es_userteam'] != ev['es_attackerteam']:
			if int(ev['hitgroup']) == 1:
				playHitsound(attacker,True)
				showHitmarker(attacker,0.2,True)
			else:
				playHitsound(attacker)
				showHitmarker(attacker,0.2)

def player_disconnect(ev):
	del players[ev['userid']]

def showHitmarker(userid,duration,headshot=False):
	if players[userid].hitmarker == None: return
	if headshot: es.cexec(userid,"r_screenoverlay %s" % markerDict[players[userid].hitmarker]['headshot'])
	else: es.cexec(userid,"r_screenoverlay %s" % markerDict[players[userid].hitmarker]['normal'])
	delay.add(userid,duration)

def playHitsound(userid,headshot=False):
	if players[userid].hitsound == None: return
	if headshot: es.playsound(userid,soundDict[players[userid].hitsound]['headshot'],players[userid].volume)
	else: es.playsound(userid,soundDict[players[userid].hitsound]['normal'],players[userid].volume)
	

def playHintSound(userid,number):
	if number == 0: es.playsound(userid,'buttons/button11.wav',1.0)
	elif number == 1: es.playsound(userid,'UI/buttonclick.wav',1.0)

def whisper(userid,msg):
	msg = '#black[#blueHit Indicator#black] %s' % str(msg)
	msg = msg.replace('#black','\x07000000')
	msg = msg.replace('#blue','\x0720B2AA')
	msg = msg.replace('#red','\x07FF3D3D')
	es.tell(userid,'#multi',msg)

def popupOpen(userid,popupname,page=None):
	if popupname == 'main':
		temp = popuplib.easymenu('popupMain',None,popupSelected)
		temp.settitle('Hit Indicator %s' % info.version)
		temp.setdescription('by Ruuubi\n-----------------------------\nHitmarker: %s\nHitsound: %s\nVolume: %s' % (players[userid].hitmarker,players[userid].hitsound,players[userid].volume))
		temp.addoption('save','Save Settings')
		temp.addoption('hitmarker','Select Hitmarker')
		temp.addoption('hitsound','Select Hitsound')
		temp.addoption('test','Test Marker+Sound')
		temp.addoption('volDown','Lower Volume')
		temp.addoption('volUp','Increase Volume')
		temp.send(userid)
		temp.delete()
	elif popupname == 'hitsound':
		temp = popuplib.easymenu('popupHitsound',None,popupSelected)
		temp.settitle('Hit Indicator %s' % info.version)
		temp.setdescription("[Select Hitsound]")
		temp.addoption((0,None),'Back to main menu\n-----------------------------')
		temp.addoption((1,None),'[X] Disable Hitsound') if players[userid].hitsound == None else temp.addoption((1,None),'[  ] Disable Hitsound')
		for idx,x in enumerate(soundDict):
			temp.addoption((idx+2,x),'[X] %s' % x) if players[userid].hitsound == x else temp.addoption((idx+2,x),'[  ] %s' % x)
		if len(temp.options['default']) < 10 and page > 1: page = 1
		temp.sendPage(userid,page)
		temp.delete()
	elif popupname == 'hitmarker':
		temp = popuplib.easymenu('popupHitmarker',None,popupSelected)
		temp.settitle('Hit Indicator %s' % info.version)
		temp.setdescription("[Select Hitmarker]")
		temp.addoption((0,None),'Back to main menu\n-----------------------------')
		temp.addoption((1,None),'[X] Disable Hitmarker') if players[userid].hitmarker == None else temp.addoption((1,None),'[  ] Disable Hitmarker')
		for idx,x in enumerate(markerDict):
			temp.addoption((idx+2,x),'[X] %s' % x) if players[userid].hitmarker == x else temp.addoption((idx+2,x),'[  ] %s' % x)
		if len(temp.options['default']) < 10 and page > 1: page = 1
		temp.sendPage(userid,page)
		temp.delete()

def popupSelected(userid,choice,popupname):
	if popupname == 'popupMain':
		if choice == 'save':
			players[userid].saveInDict()
			whisper(userid,'#GreenSaved settings!')
			playHintSound(userid,1)
		elif choice == 'hitmarker':
			popupOpen(userid,'hitmarker',1)
			playHintSound(userid,1)
		elif choice == 'hitsound':
			popupOpen(userid,'hitsound',1)
			playHintSound(userid,1)
		elif choice == 'test':
			showHitmarker(userid,1)
			playHitsound(userid)
			popupOpen(userid,'main')
		elif choice == 'volDown':
			players[userid].volume = max(players[userid].volume-0.1,0.1)
			playHitsound(userid)
			popupOpen(userid,'main')
		elif choice == 'volUp':
			players[userid].volume = min(players[userid].volume+0.1,1.0)
			playHitsound(userid)
			popupOpen(userid,'main')
	elif popupname == 'popupHitmarker':
		if choice[0] > 0:
			players[userid].hitmarker = choice[1]
			showHitmarker(userid,1)
			page = divmod(choice[0],7)[0]+1
			popupOpen(userid,'hitmarker',page)
		else:
			playHintSound(userid,1)
			popupOpen(userid,'main')
	elif popupname == 'popupHitsound':
		if choice[0] > 0:
			players[userid].hitsound = choice[1]
			playHitsound(userid)
			page = divmod(choice[0],7)[0]+1
			popupOpen(userid,'hitsound',page)
		else:
			playHintSound(userid,1)
			popupOpen(userid,'main')